/*
 * Copyright (c) 2021 Espressif Systems (Shanghai) Co., Ltd.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#define DT_DRV_COMPAT nordic_qspicustom

#define FLASH_WRITE_BLK_SZ 8
#define FLASH_ERASE_BLK_SZ 8
/*
 * HAL includes go first to
 * avoid BIT macro redefinition
 */

#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <zephyr/drivers/flash.h>
#include <soc.h>

#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(flash_tester);

struct flash_tester_dev_config
{
    void *controller;
};

struct flash_tester_dev_data
{
    struct k_sem sem;
};

static const struct flash_parameters flash_tester_parameters = {
    .write_block_size = FLASH_WRITE_BLK_SZ,
    .erase_value = 0xff,
};

static int flash_tester_read(const struct device *dev, off_t address, void *buffer, size_t length)
{
    int ret = 0;

    printk("flash_tester_read");
    return ret;
}

static int flash_tester_write(const struct device *dev,
                              off_t address,
                              const void *buffer,
                              size_t length)
{
    int ret = 0;
    printk("flash_tester_write");
    return ret;
}

static int flash_tester_erase(const struct device *dev, off_t start, size_t len)
{
    int ret = 0;
    printk("flash_tester_erase");
    return ret;
}

#if CONFIG_FLASH_PAGE_LAYOUT
static const struct flash_pages_layout flash_tester_pages_layout = {
    .pages_count = 32,
    .pages_size = 128,
};

void flash_tester_page_layout(const struct device *dev,
                              const struct flash_pages_layout **layout,
                              size_t *layout_size)
{
    *layout = &flash_tester_pages_layout;
    *layout_size = 1;
}
#endif /* CONFIG_FLASH_PAGE_LAYOUT */

static const struct flash_parameters *
flash_tester_get_parameters(const struct device *dev)
{
    ARG_UNUSED(dev);

    return &flash_tester_parameters;
}

static int flash_tester_init(const struct device *dev)
{
    printk("flash_tester_init");

    return 0;
}

static const struct flash_driver_api flash_tester_driver_api = {
    .read = flash_tester_read,
    .write = flash_tester_write,
    .erase = flash_tester_erase,
    .get_parameters = flash_tester_get_parameters,
#ifdef CONFIG_FLASH_PAGE_LAYOUT
    .page_layout = flash_tester_page_layout,
#endif
};

static struct flash_tester_dev_data flash_tester_data;

static const struct flash_tester_dev_config flash_tester_config = {
    .controller = (void *)DT_INST_REG_ADDR(0),
};

DEVICE_DT_INST_DEFINE(0, flash_tester_init,
                      NULL,
                      &flash_tester_data, &flash_tester_config,
                      POST_KERNEL, CONFIG_FLASH_INIT_PRIORITY,
                      &flash_tester_driver_api);